#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 Elias Soong <elias.soong@gmail.com>
# All rights reserved.
#
# http://www.elias.cn/.
#
# Author: Elias Soong <elias.soong@gmail.com>

import os.path, os
import time
import tarfile
import sys, getopt

__version__ = '0.1'

STAMP_FILE_NAME = sys.path[0] + '/.mdb_stamp'
SENDMAIL_PATH = '/usr/sbin/sendmail'
HELP_STRING = 'python MailDiffBackup.py [-h] [-s SourceDirectory] [-t TempDirectory] [-p PreFix] [-m MailAddress]'

sourceDir = '.'
tmpDir = '.'
prefix = 'backup'
mail = 'root@localhost'
nowTime = time.time()
timeThres = 0
newFilesList = []

def getNewFiles(timeThres, dir, files):
    """Record files those are newer than timeThres. """ 
    for file in files:
        path = dir + '/' + file
        if os.path.isfile(path) and os.path.getmtime(path) > timeThres:
            newFilesList.append(path)

def createTarFile():
    """Archive updated files into single tar file."""
    tarName = prefix + time.strftime('_%Y_%m_%d_%H_%M.tar.gz', time.gmtime(nowTime))
    tar = tarfile.open(tmpDir + '/' + tarName, "w:gz")
    for file in newFilesList:
        name = file.replace(sourceDir, '', 1)
        tar.add(file,name)
    tar.close()

def sendMail():
    """Send the newest tar file through "sendmail" program."""
    tarName = prefix + time.strftime('_%Y_%m_%d_%H_%M.tar.gz', time.gmtime(nowTime))
    tarPath = tmpDir + '/' + tarName
    p = os.popen('cat %s | uuencode %s | %s %s' % (tarPath, tarPath, SENDMAIL_PATH, mail))
    sts = p.close()
    return sts and sts or 0

def mailDiffBackup():
    """Archive new files as a tar file, and email this tar file out."""
    os.path.walk(sourceDir, getNewFiles, timeThres)
    sts = -1
    if len(newFilesList) > 0:
        createTarFile()
        sts = sendMail()
    if sts == 0:
        timeStamp = time.ctime(nowTime)
        stampFile = open(STAMP_FILE_NAME,"w")
        stampFile.write(timeStamp)
        stampFile.close()

if __name__ == '__main__':
    sourceDir = 'D:/tmp/Django-0.96'
    if os.path.exists(STAMP_FILE_NAME):
        stampFile = open(STAMP_FILE_NAME)
        stampString = stampFile.readline()
        timeThres = time.mktime(time.strptime(stampString))
        stampFile.close()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hs:t:p:m:", ["help"])
    except getopt.GetoptError:
        print HELP_STRING
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            print HELP_STRING
            sys.exit()
        elif opt == '-s':
            sourceDir = arg
        elif opt == '-t':
            tmpDir = arg
        elif opt == '-p':
            prefix = arg
        elif opt == '-m':
            mail = arg
    mailDiffBackup()
    
